#!/usr/bin/env python

# OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv
# This is a standalone program. Pass an image name as a first parameter of the program.

import sys
from math import sin, cos, sqrt
from pyopencv import *

# toggle between CV_HOUGH_STANDARD and CV_HOUGH_PROBILISTIC
USE_STANDARD=0

if __name__ == "__main__":
    filename = "building.jpg"
    if len(sys.argv)>1:
        filename = sys.argv[1]

    src=imread(filename, 0)
    if src.empty():
        print("Error opening image %s" % filename)
        sys.exit(-1)

    dst = Mat(src.size(), CV_8UC1)
    color_dst = Mat(src.size(), CV_8UC3)
    lines = 0
    Canny( src, dst, 50, 200, 3 )
    cvtColor( dst, color_dst, CV_GRAY2BGR )

    if USE_STANDARD:
        lines = HoughLines( dst, 1, CV_PI/180, 100, 0, 0 )

        for i in range(min(len(lines), 100)):
            l = lines[i]
            rho = l[0]
            theta = l[1]
            a = cos(theta)
            b = sin(theta)
            x0 = a*rho 
            y0 = b*rho
            pt1 = Point(round(x0 + 1000*(-b)), round(y0 + 1000*(a)))
            pt2 = Point(round(x0 - 1000*(-b)), round(y0 - 1000*(a)))
            line( color_dst, pt1, pt2, CV_RGB(255,0,0), 3, 8 )

    else:
        lines = HoughLinesP( dst, 1, CV_PI/180, 50, 50, 10 )
        for l in lines:
            line( color_dst, Point(int(l[0]), int(l[1])), Point(int(l[2]), int(l[3])), CV_RGB(255,0,0), 3, 8 )

    namedWindow( "Source", 1 )
    imshow( "Source", src )

    namedWindow( "Hough", 1 )
    imshow( "Hough", color_dst )

    waitKey(0)



#!/usr/bin/env python

# OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv
from pyopencv import *
import numpy.random as NR
MAX_CLUSTERS=5

if __name__ == "__main__":

    color_tab = [CV_RGB(255,0,0),CV_RGB(0,255,0),CV_RGB(100,100,255), CV_RGB(255,0,255),CV_RGB(255,255,0)]
    img = Mat(Size(500, 500), CV_8UC3)
    rng = RNG()
    namedWindow( "clusters", 1 )
        
    while True:
        cluster_count = rng.as_unsigned()%(MAX_CLUSTERS-1) + 2
        
        # generate random sample from multigaussian distribution
        points = NR.randn(cluster_count, rng.as_unsigned()%200 + 1, 2)*(img.cols, img.rows)*0.1
        for k in range(cluster_count):
            points[k] += (rng.as_unsigned()%img.cols, rng.as_unsigned()%img.rows)
        sample_count = points.size/2
        points = asMat(points.reshape(sample_count, 1, 2).astype('float32'))
        randShuffle( points )
        
        # K Means Clustering
        clusters = Mat(points.size(), CV_32SC1)
        compact, centers = kmeans(points, cluster_count, clusters, 
            TermCriteria(TermCriteria.EPS+TermCriteria.MAX_ITER, 10, 1.0), 1, KMEANS_RANDOM_CENTERS)

        img.setTo(0)
        pts = points[:].reshape(sample_count, 2).astype('int32')
        for i in range(sample_count):
            circle(img, asPoint(pts[i]), 2, color_tab[clusters[i,0]], CV_FILLED, CV_AA, 0)
        
        imshow( "clusters", img )

        if '%c' % (waitKey(0) & 255) in ['\x1b','q','Q']: # 'ESC'
            break



#!/usr/bin/env python

# OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv
#
# The full "Square Detector" program.
# It loads several images subsequentally and tries to find squares in
# each image
#

from pyopencv import *
from math import sqrt
import numpy as NP
import numpy.linalg as NL

thresh = 50
img = Mat()
img0 = Mat()
wndname = "Square Detection Demo"

def angle( pt1, pt2, pt0 ):
    d1 = pt1[:] - pt0[:]
    d2 = pt2[:] - pt0[:]
    return NP.dot(d1, d2)/(NL.norm(d1)*NL.norm(d2)+1e-5)

def findSquares4( img, thresh ):
    N = 11
    sz = Size( img.cols & -2, img.rows & -2 )
    timg = img.clone() # make a copy of input image
    gray = Mat(sz, CV_8UC1)
    pyr = Mat( Size(int(sz.width/2), int(sz.height/2)), CV_8UC3 )
    # create empty sequence that will contain points -
    # 4 points per square (the square's vertices)
    squares = []

    # select the maximum ROI in the image
    # with the width and height divisible by 2
    subimage = Mat(sz, img.type())

    # down-scale and upscale the image to filter out the noise
    pyrDown( timg, pyr, pyr.size() )
    pyrUp( pyr, subimage, subimage.size() )
    # extract the color planes
    tgrays = [Mat(sz, CV_8UC1) for i in range(3)]
    channels = [Mat(sz, CV_8UC1) for i in range(3)]
    split( timg, tgrays )
    split( subimage, channels ) 
    # find squares in every color plane of the image
    for c in range(3):
        for l in range(N):
            # hack: use Canny instead of zero threshold level.
            # Canny helps to catch squares with gradient shading
            if( l == 0 ):
                # apply Canny. Take the upper threshold from slider
                # and set the lower to 0 (which forces edges merging)
                Canny( tgrays[c], gray, 0, thresh, 5 )
                # dilate canny output to remove potential
                # holes between edge segments
                dilate( gray, gray, Mat() )
            else:
                # apply threshold if l!=0:
                #     tgray(x,y) = gray(x,y) < (l+1)*255/N ? 255 : 0
                threshold( tgrays[c], gray, (l+1)*255/N, 255, THRESH_BINARY )

            # find contours and store them all as a list
            contours, hierarchy = findContours(gray, mode=RETR_LIST, method=CHAIN_APPROX_SIMPLE)

            if len(contours) == 0:
                continue
            
            # test each contour
            for contour in contours:
                # approximate contour with accuracy proportional
                # to the contour perimeter
                mat_contour = asMat(contour)
                result = approxPolyDP( mat_contour, arcLength(mat_contour, False)*0.02, False )
                mat_result = asMat(result)
                # square contours should have 4 vertices after approximation
                # relatively large area (to filter out noisy contours)
                # and be convex.
                # Note: absolute value of an area is used because
                # area may be positive or negative - in accordance with the
                # contour orientation
                if( len(result) == 4 and 
                    abs(contourArea(mat_result)) > 1000 and 
                    isContourConvex(mat_result) ):
                    s = 0;
                    for i in range(4):
                        # find minimum angle between joint
                        # edges (maximum of cosine)
                        t = abs(angle( result[i], result[i-2], result[i-1]))
                        if s<t:
                            s=t
                    # if cosines of all angles are small
                    # (all angles are ~90 degree) then write quandrange
                    # vertices to resultant sequence
                    if( s < 0.3 ):
                        squares.append(result)

    return squares

def on_trackbar( a, param ):
    if not img.empty():
        squares = findSquares4( img, a )
        cpy = img.clone()        
        # draw the squares as a closed polylines
        polylines( cpy, squares, 1, CV_RGB(0,255,0), 3, CV_AA, 0 );
        # show the resultant image
        imshow( wndname, cpy );

names =  ["pic1.png", "pic2.png", "pic3.png", "pic4.png", "pic5.png", "pic6.png" ];

if __name__ == "__main__":
    for name in names:
        img0 = imread( name, 1 )
        if img0.empty():
            print("Couldn't load %s" % name)
            continue
        img = img0.clone()
        # create window and a trackbar (slider) with parent "image" and set callback
        # (the slider regulates upper threshold, passed to Canny edge detector)
        namedWindow( wndname, 1 )
        createTrackbar( "canny thresh", wndname, thresh, 1000, on_trackbar )
        # force the image processing
        on_trackbar(0, None)
        # wait for key.
        # Also the function cvWaitKey takes care of event processing
        c = waitKey(0)
        if( c & 255 == 27 ):
            break



#!/usr/bin/env python

# OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv
import sys
from pyopencv import *
from ctypes import c_int
image0 = None
image1 = None
level = 4
block_size = 1000
storage = None

def ON_SEGMENT(pos, param):
    comp = pyrSegmentation(image0, image1, storage, level, threshold1.value+1, threshold2.value+1)
    imshow("Segmentation", image1)

if __name__ == "__main__":
    filename = "fruits.jpg";
    if len(sys.argv) == 2:
        filename = sys.argv[1]
    image = imread( filename, 1)
    if image.empty():
        print("Error opening %s" % filename)
        sys.exit(-1)

    namedWindow("Source", 0)
    imshow("Source", image)
    namedWindow("Segmentation", 0)
    storage = createMemStorage ( block_size )
    image.cols &= -(1<<level)
    image.rows &= -(1<<level)
    image0 = image.clone()
    image1 = image.clone()
    # segmentation of the color image
    l = 1
    threshold1 = c_int(255)
    threshold2 = c_int(30)
    ON_SEGMENT(1, None)
    sthreshold1 = createTrackbar("Threshold1", "Segmentation", threshold1, 255, ON_SEGMENT)
    sthreshold2 = createTrackbar("Threshold2", "Segmentation",  threshold2, 255, ON_SEGMENT)
    imshow("Segmentation", image1)
    waitKey(0)



#!/usr/bin/env python

# OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv
from pyopencv import *
from ctypes import c_int
import sys

sigma = c_int(3)
smoothType = CV_GAUSSIAN

if __name__ == "__main__":
    laplace = None
    colorlaplace = None
    planes = None
    capture = None
    
    if len(sys.argv)==1:
        capture = VideoCapture( 0 )
    elif len(sys.argv)==2 and sys.argv[1].isdigit():
        capture = VideoCapture( int(sys.argv[1]) )
    elif len(sys.argv)==2:
        capture = VideoCapture( sys.argv[1] ); 

    if not capture.isOpened():
        print("Could not initialize capturing...")
        sys.exit(-1)
        
    namedWindow( "Laplacian", CV_WINDOW_AUTOSIZE )
    createTrackbar( "Sigma", "Laplacian", sigma, 15 )
    frame = Mat()

    while True:
        capture >> frame
        if frame.empty():
            break
            
        if not planes:
            planes = [Mat(frame.size(), CV_8UC1) for i in range(3)]

        if not laplace:
            laplace = Mat(frame.size(), CV_16SC1)
            colorlaplace = Mat(frame.size(), CV_8UC3)

        ksize = (sigma.value*5)|1
        smooth( frame, colorlaplace, smoothType, ksize, ksize, sigma.value, sigma.value )        
        split( colorlaplace, planes )
        for plane in planes:
            Laplacian( plane, laplace, 3 )
            convertScaleAbs( laplace, plane, 1, 0 )

        merge( planes, colorlaplace )

        imshow("Laplacian", colorlaplace )

        c = '%c' % (waitKey(10) & 255)
        if c == ' ':
            smoothType = CV_BLUR if smoothType == CV_GAUSSIAN else CV_MEDIAN if smoothType == CV_BLUR else CV_GAUSSIAN
        elif c in ['\x1b','q','Q']: # 'ESC'
            break
            


#!/usr/bin/env python

# OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv
from pyopencv import *
from ctypes import c_int
import sys

if __name__ == "__main__":
    img = Mat()
    if len(sys.argv) > 1:
        img = imread(sys.argv[1])
        
    if img.empty():
        print( "ERROR: no image was specified\n" if len(sys.argv) == 1 else "ERROR: the specified image could not be loaded\n")
        print( "Usage: peopledetect <inputimage>\n" )
        sys.exit(-1)

    hog = HOGDescriptor()
    hog.setSVMDetector(HOGDescriptor.getDefaultPeopleDetector())
    t = getTickCount()
    # run the detector with default parameters. to get a higher hit-rate
    # (and more false alarms, respectively), decrease the hitThreshold and
    # groupThreshold (set groupThreshold to 0 to turn off the grouping completely).
    found = hog.detectMultiScale(img, 0, Size(8,8), Size(24,16), 1.05, 2)
    t = float(getTickCount()) - t
    print("Detection time = %gms\n" % (t*1000./getTickFrequency()))
    for r in found:
        # the HOG detector returns slightly larger rectangles than the real objects.
        # so we slightly shrink the rectangles to get a nicer output.
        r.x += round(r.width*0.1)
        r.y += round(r.height*0.1)
        r.width = round(r.width*0.8)
        r.height = round(r.height*0.8)
        rectangle(img, r.tl(), r.br(), Scalar(0,255,0), 1)

    namedWindow("people detector", 1)
    imshow("people detector", img)
    waitKey(0)



#!/usr/bin/env python

# OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv
import sys
from pyopencv import *

src=None
dst=None
src2=None

def on_mouse( event, x, y, flags, param ):

    if( not src ):
        return;

    if event==CV_EVENT_LBUTTONDOWN:
        logPolar( src, dst, Point2f(x,y), 40, INTER_LINEAR+CV_WARP_FILL_OUTLIERS )
        logPolar( dst, src2, Point2f(x,y), 40, INTER_LINEAR+CV_WARP_FILL_OUTLIERS+CV_WARP_INVERSE_MAP )
        imshow( "log-polar", dst )
        imshow( "inverse log-polar", src2 )

if __name__ == "__main__":
    
    filename = "fruits.jpg"
    if len(sys.argv)>1:
        filename=argv[1]
    
    src = imread(filename, CV_LOAD_IMAGE_COLOR)
    if not src:
        print("Could not open %s" % filename)
        sys.exit(-1)
        
    namedWindow( "original",1 )
    namedWindow( "log-polar", 1 )
    namedWindow( "inverse log-polar", 1 )
  
    
    dst = Mat(Size(256,256), CV_8UC3 )
    src2 = Mat(src.size(), CV_8UC3 )
    
    setMouseCallback( "original", on_mouse )
    on_mouse( CV_EVENT_LBUTTONDOWN, src.cols/2, src.rows/2, None, None)
    
    imshow( "original", src )
    waitKey()



#!/usr/bin/env python

# OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv
import sys
from pyopencv import *


if __name__ == "__main__":
    
    log_polar_img = None
    lin_polar_img = None
    recovered_img = None

    if len(sys.argv)==1:
        capture = VideoCapture( 0 )
    elif len(sys.argv)==2 and sys.argv[1].isdigit():
        capture = VideoCapture( int(sys.argv[1]) )
    elif len(sys.argv)==2:
        capture = VideoCapture( sys.argv[1] );

    if not capture.isOpened():
        print("Could not initialize capturing...")
        print("Usage: %s <CAMERA_NUMBER>    , or \n       %s <VIDEO_FILE>\n" % (sys.argv[0],sys.argv[0]))
        sys.exit(-1)
        
    frame = Mat()
        
    namedWindow( "Linear-Polar", 0 )
    namedWindow( "Log-Polar", 0 )
    namedWindow( "Recovered image", 0 )

    moveWindow( "Linear-Polar", 20,20 )
    moveWindow( "Log-Polar", 700,20 )
    moveWindow( "Recovered image", 20,700 )

    while True:
        capture >> frame
        if frame.empty():
            break
            
        if log_polar_img is None:
            log_polar_img = Mat(frame.size(), CV_MAKETYPE(CV_8U, frame.channels()))
            lin_polar_img = Mat(frame.size(), CV_MAKETYPE(CV_8U, frame.channels()))
            recovered_img = Mat(frame.size(), CV_MAKETYPE(CV_8U, frame.channels()))

        pt0 = Point2f(0.5*frame.cols, 0.5*frame.rows)
        logPolar(frame,log_polar_img,pt0,70, INTER_LINEAR+CV_WARP_FILL_OUTLIERS)
        linearPolar(frame,lin_polar_img,pt0,70, INTER_LINEAR+CV_WARP_FILL_OUTLIERS)

        if False: # change this to True to see a different effect
            logPolar(log_polar_img,recovered_img,pt0,70, CV_WARP_INVERSE_MAP+INTER_LINEAR)
        else:
            linearPolar(lin_polar_img,recovered_img,pt0,70, CV_WARP_INVERSE_MAP+INTER_LINEAR+CV_WARP_FILL_OUTLIERS)

        imshow("Log-Polar", log_polar_img );
        imshow("Linear-Polar", lin_polar_img );
        imshow("Recovered image", recovered_img );

        if waitKey(10) >= 0:
            break



#!/usr/bin/env python

# OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv
from pyopencv import *
from math import cos, sin

w = 500
levels = 3
contours = None
hierarchy = None

def on_trackbar(pos, param):
    cnt_img = Mat(Size(w, w), CV_8UC3)
    cnt_img.setTo(0)
    
    if pos <= 3:
        idx = int(hierarchy[hierarchy[hierarchy[0,0],0],0])
        max_level = 3-pos
    else:
        idx = -1
        max_level = pos-3
    
    drawContours( cnt_img, contours, idx, CV_RGB(255,0,0), CV_FILLED, 8, hierarchy, max_level )
    imshow( "contours", cnt_img )
    
def generate_image():    
    img = Mat(Size(w,w), CV_8UC1)
    img.setTo(0)

    for i in range(6):
        dx = (i%2)*250 - 30
        dy = (i/2)*150
        white = Scalar(255)
        black = Scalar(0)

        if i==0:
            for j in range(11):
                angle = (j+5)*CV_PI/21
                line(img, Point(round(dx+100+j*10-80*cos(angle)),
                    round(dy+100-90*sin(angle))),
                    Point(round(dx+100+j*10-30*cos(angle)),
                    round(dy+100-30*sin(angle))), white, 1, 8, 0)

        ellipse( img, Point(dx+150, dy+100), Size(100,70), 0, 0, 360, white, -1, 8, 0 )
        ellipse( img, Point(dx+115, dy+70), Size(30,20), 0, 0, 360, black, -1, 8, 0 )
        ellipse( img, Point(dx+185, dy+70), Size(30,20), 0, 0, 360, black, -1, 8, 0 )
        ellipse( img, Point(dx+115, dy+70), Size(15,15), 0, 0, 360, white, -1, 8, 0 )
        ellipse( img, Point(dx+185, dy+70), Size(15,15), 0, 0, 360, white, -1, 8, 0 )
        ellipse( img, Point(dx+115, dy+70), Size(5,5), 0, 0, 360, black, -1, 8, 0 )
        ellipse( img, Point(dx+185, dy+70), Size(5,5), 0, 0, 360, black, -1, 8, 0 )
        ellipse( img, Point(dx+150, dy+100), Size(10,5), 0, 0, 360, black, -1, 8, 0 )
        ellipse( img, Point(dx+150, dy+150), Size(40,10), 0, 0, 360, black, -1, 8, 0 )
        ellipse( img, Point(dx+27, dy+100), Size(20,35), 0, 0, 360, white, -1, 8, 0 )
        ellipse( img, Point(dx+273, dy+100), Size(20,35), 0, 0, 360, white, -1, 8, 0 )
        
    return img

if __name__=='__main__':
    img = generate_image()
        
    namedWindow( "image", 1 )
    imshow( "image", img )

    contours, hierarchy = findContours( img, RETR_TREE, CHAIN_APPROX_SIMPLE, Point(0,0) )

    # comment this out if you do not want approximation
    contours = [approxPolyDP(asMat(x.reshape(1,len(x),2)), 3, 1) for x in contours]

    namedWindow( "contours", 1 )
    createTrackbar( "levels+3", "contours", levels, 7, on_trackbar )

    on_trackbar(0, None)
    waitKey(0)



#!/usr/bin/env python

# OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv
from pyopencv import *
import sys

if __name__ == "__main__":
    filename = sys.argv[1]
    im = imread(filename, CV_LOAD_IMAGE_GRAYSCALE)
    im3 = imread(filename, CV_LOAD_IMAGE_COLOR)
    chessboard_dim = Size( 7, 5 )
    
    found, corners = findChessboardCorners( im, chessboard_dim )
    drawChessboardCorners( im3, chessboard_dim, corners, found )
    
    imshow("win", im3)
    waitKey()



#!/usr/bin/env python
# OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv

print("OpenCV Python version of convexhull")

# import the necessary things for OpenCV
from pyopencv import *

# to generate random values
import random

# how many points we want at max
_MAX_POINTS = 100

if __name__ == '__main__':

    # main object to get random values from
    my_random = random.Random ()

    # create the image where we want to display results
    image = Mat(Size(500, 500), CV_8UC3)

    while True:
        # do forever

        # get a random number of points
        count = my_random.randrange (0, _MAX_POINTS) + 1

        # initialisations
        points = Mat(Size(count, 1), CV_32SC2)
        
        for i in range (count):
            # generate a random point
            points[0,i] = (my_random.randrange (0, image.cols / 2) + image.cols / 4,
                my_random.randrange (0, image.cols / 2) + image.cols / 4)

        # compute the convex hull
        hull = convexHull(points)

        # start with an empty image
        image[:] = 0

        for i in range (count):
            # draw all the points
            circle (image, asPoint(points[0,i]), 2, Scalar (0, 0, 255), CV_FILLED, CV_AA, 0)

        # start the line from the last point
        pt0 = hull[-1]
        
        for pt1 in hull:
            # draw
            line (image, asPoint(pt0), asPoint(pt1), Scalar (0, 255, 0), 1, CV_AA, 0)

            # now, current one will be the previous one for the next iteration
            pt0 = pt1

        # display the final image
        imshow ('hull', image)

        # handle events, and wait a key pressed
        if waitKey (0) & 255 == 27:
            # user has press the ESC key, so exit
            break



#!/usr/bin/env python

# OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv
from pyopencv import *
import numpy.random as NR
import sys

marker_mask = None
markers = None
img0 = None
img = None
img_gray = None 
wshed = None
prev_pt = Point(-1,-1)

def on_mouse( event, x, y, flags, param ):
    global prev_pt
    if img.empty():
        return;
    if( event == CV_EVENT_LBUTTONUP or not (flags & CV_EVENT_FLAG_LBUTTON) ):
        prev_pt = Point(-1,-1)
    elif( event == CV_EVENT_LBUTTONDOWN ):
        prev_pt = Point(x,y)
    elif( event == CV_EVENT_MOUSEMOVE and (flags & CV_EVENT_FLAG_LBUTTON) ):
        pt = Point(x,y)
        if prev_pt.x < 0:
            prev_pt = pt
        line( marker_mask, prev_pt, pt, Scalar.all(255), 5, 8, 0 )
        line( img, prev_pt, pt, Scalar.all(255), 5, 8, 0 )
        prev_pt = pt
        imshow( "image", img )

if __name__ == "__main__":
    filename = "fruits.jpg"
    if len(sys.argv)>1:
        filename = sys.argv[1]

    img0 = imread(filename,1)
    if img0.empty():
        print("Error opening image '%s'" % filename)
        sys.exit(-1)

    print("Hot keys:")
    print("\tESC - quit the program")
    print("\tr - restore the original image")
    print("\tw - run watershed algorithm")
    print("\t  (before that, roughly outline several markers on the image)")

    namedWindow( "image", 1 )
    namedWindow( "watershed transform", 1 )

    img = img0.clone()
    img_gray = img0.clone()
    wshed = img0.clone()
    marker_mask = Mat(img.size(), CV_8UC1)
    markers = Mat(img.size(), CV_32SC1, 0)
    markers2 = Mat(img.size(), CV_8UC1)

    # color table for LUT
    color_tab = asMat((NR.rand(1, 256, 3)*180+50).astype('uint8'))

    cvtColor( img, marker_mask, CV_BGR2GRAY )
    cvtColor( marker_mask, img_gray, CV_GRAY2BGR )

    marker_mask.setTo(0)
    wshed.setTo(0)

    imshow( "image", img )
    imshow( "watershed transform", wshed )

    setMouseCallback( "image", on_mouse )
    while True:
        c = '%c' % (waitKey(0) & 255)
        if c=='\x1b':
            break;
        elif c == 'r':
            marker_mask.setTo(0)
            markers.setTo(0)
            img0.copyTo(img)
            imshow( "image", img )
        elif c == 'w':
            contours, hierarchy = findContours(marker_mask, mode=RETR_CCOMP, method=CHAIN_APPROX_SIMPLE)
            comp_count = 0
            idx = 0
            while idx >= 0:
                # there's a bug with not specifying maxLevel
                drawContours( markers, contours, idx, comp_count+1, CV_FILLED, 8, hierarchy, 0 )
                idx = int(hierarchy[idx][0])
                comp_count += 1
                
            t = getTickCount()
            watershed( img0, markers )
            t = getTickCount() - t
            print("exec time = %lf" % (t/(getTickFrequency()*1000.)))

            wshed.setTo(Scalar.all(255))

            # paint the watershed image
            markers.convertTo(markers2, CV_8UC1)
            cvtColor( markers2, wshed, CV_GRAY2BGR )
            LUT(wshed, color_tab, wshed)

            addWeighted( wshed, 0.5, img_gray, 0.5, 0, wshed )
            imshow( "watershed transform", wshed )



#!/usr/bin/env python

# OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv
import sys
from pyopencv import *

cascadeName = "haarcascade_frontalface_alt.xml"
nestedCascadeName = "haarcascade_eye_tree_eyeglasses.xml"

def detectAndDraw(img, cascade, nestedCascade, scale):
    i = 0
    t = 0.0
    colors =  ( CV_RGB(0,0,255),
        CV_RGB(0,128,255),
        CV_RGB(0,255,255),
        CV_RGB(0,255,0),
        CV_RGB(255,128,0),
        CV_RGB(255,255,0),
        CV_RGB(255,0,0),
        CV_RGB(255,0,255))
    gray = Mat()
    smallImg = Mat( Size(round(img.cols/scale), round(img.rows/scale)), CV_8UC1 )

    cvtColor( img, gray, CV_BGR2GRAY )
    resize( gray, smallImg, smallImg.size(), 0, 0, INTER_LINEAR )
    equalizeHist( smallImg, smallImg )

    t = getTickCount()
    faces = cascade.detectMultiScale( smallImg, 
        1.1, 2, 0
        #|CascadeClassifier.FIND_BIGGEST_OBJECT
        #|CascadeClassifier.DO_ROUGH_SEARCH
        |CascadeClassifier.SCALE_IMAGE
        ,
        Size(30, 30) )
    t = getTickCount() - t
    print( "detection time = %lf ms\n" % (t/(getTickFrequency()*1000.)) )
    for i in range(len(faces)):
        r = faces[i]
        color = colors[i%8]
        center = Point(round((r.x + r.width*0.5)*scale), round((r.y + r.height*0.5)*scale))
        radius = round((r.width + r.height)*0.25*scale)
        circle( img, center, radius, color, 3, 8, 0 )
        if nestedCascade.empty():
            continue
        smallImgROI = smallImg(r)
        nestedObjects = nestedCascade.detectMultiScale( smallImgROI, 
            1.1, 2, 0
            #|CascadeClassifier.FIND_BIGGEST_OBJECT
            #|CascadeClassifier.DO_ROUGH_SEARCH
            #|CascadeClassifier.DO_CANNY_PRUNING
            |CascadeClassifier.SCALE_IMAGE
            ,
            Size(30, 30) )
        for nr in nestedObjects:
            center = Point(round((r.x + nr.x + nr.width*0.5)*scale), round((r.y + nr.y + nr.height*0.5)*scale))
            radius = round((nr.width + nr.height)*0.25*scale)
            circle( img, center, radius, color, 3, 8, 0 )
    imshow( "result", img )

if __name__ == '__main__':
    capture = VideoCapture()
    frame = Mat()
    image = Mat()
    scaleOpt = "--scale="
    cascadeOpt = "--cascade="
    nestedCascadeOpt = "--nested-cascade"
    nestedCascadeOptLen = len(nestedCascadeOpt)
    inputName = ""

    cascade = CascadeClassifier()
    nestedCascade = CascadeClassifier()
    scale = 1.0

    for i in range(len(sys.argv)):
        if sys.argv[i].startswith(cascadeOpt):
            cascadeName = sys.argv[i][len(cascadeOpt):]
        elif sys.argv[i].startswith(nestedCascadeOpt):
            if sys.argv[i][nestedCascadeOptLen] == '=':
                nestedCascadeName = sys.argv[i][nestedCascadeOptLen+1:]
            if not nestedCascade.load( nestedCascadeName ):
                print("WARNING: Could not load classifier cascade for nested objects")
        elif sys.argv[i].startswith(scaleOpt):
            try:
                scale = float(sys.argv[i][len(scaleOpt):])
                if scale < 1:
                    raise ValueError()
            except:
                scale = 1.0
        elif sys.argv[i][0] == '-':
            print("WARNING: Unknown option %s" % sys.argv[i])
        else:
            inputName = sys.argv[i]

    if not cascade.load( cascadeName ):
        print("ERROR: Could not load classifier cascade")
        print("Usage: facedetect [--cascade=\"<cascade_path>\"]\n" \
            "   [--nested-cascade[=\"nested_cascade_path\"]]\n" \
            "   [--scale[=<image scale>\n" \
            "   [filename|camera_index]\n")
        sys.exit(-1)

    if not inputName or inputName.isdigit():
        capture.open( int(inputName) )
    elif inputName:
        image = imread( inputName, 1 )
        if image.empty():
            capture.open( inputName )
    else:
        image = imread( "lena.jpg", 1 )

    namedWindow( "result", 1 )

    if capture.isOpened():
        while True:
            capture >> frame
            if frame.empty():
                break

            detectAndDraw( frame, cascade, nestedCascade, scale )

            if waitKey( 10 ) >= 0:
                break
    else:
        if not image.empty():
            detectAndDraw( image, cascade, nestedCascade, scale )
            waitKey(0)
        elif inputName:
            # assume it is a text file containing the
            # list of the image filenames to be processed - one per line
            try:
                f = open( inputName, "rt" )
                for buf in f.readlines():
                    buf = buf.rstrip()
                    print("file %s" % buf)
                    image = imread( buf, 1 )
                    if not image.empty():
                        detectAndDraw( image, cascade, nestedCascade, scale )
                        if '%c' % (waitKey(0) & 255) in ['\x1b','q','Q']: # 'ESC'
                            break;
            except IOError:
                pass



#!/usr/bin/env python
# OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv

print("OpenCV Python version of edge")

import sys

# import the necessary things for OpenCV
from pyopencv import *

# some definitions
win_name = "Edge"
trackbar_name = "Threshold"

# the callback on the trackbar
def on_trackbar (position, param):
    smooth (gray, edge, CV_BLUR, 3, 3, 0)
    bitwise_not (gray, edge)

    # run the edge dector on gray scale
    Canny (gray, edge, position, position * 3, 3)

    # reset
    col_edge[:] = 0

    # copy edge points
    image.copyTo(col_edge, edge)
    
    # show the image
    imshow (win_name, col_edge)

if __name__ == '__main__':
    filename = "fruits.jpg"

    if len(sys.argv)>1:
        filename = sys.argv[1]

    # load the image gived on the command line
    image = imread (filename)

    if not image:
        print("Error loading image '%s'" % filename)
        sys.exit(-1)

    # create the output image
    col_edge = Mat(image.size(), CV_8UC3)

    # convert to grayscale
    gray = Mat(image.size(), CV_8UC1)
    edge = Mat(image.size(), CV_8UC1)
    cvtColor (image, gray, CV_BGR2GRAY)

    # create the window
    namedWindow (win_name, CV_WINDOW_AUTOSIZE)

    # create the trackbar
    createTrackbar (trackbar_name, win_name, 1, 100, on_trackbar)

    # show the image
    on_trackbar (0, None)

    # wait a key pressed to end
    waitKey(0)



#!/usr/bin/env python

# Old-style OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv
import sys
from pyopencv import *

wndname = "Distance transform"
tbarname = "Threshold"

# The output images
dist = None
dist8u1 = None
dist8u2 = None
dist8u = None
dist32s = None

gray = None
edge = None

# define a trackbar callback
def on_trackbar( edge_thresh, param ):

    threshold( gray, edge, edge_thresh, edge_thresh, THRESH_BINARY )
    #Distance transform                  
    distanceTransform( edge, dist, CV_DIST_L2, CV_DIST_MASK_5 )

    dist.convertTo(dist, dist.type(), 5000.0, 0 )
    pow( dist, 0.5, dist )
    
    dist.convertTo( dist32s, dist32s.type(), 1.0, 0.5 )
    bitwise_and( dist32s, Scalar.all(255), dist32s )
    dist32s.convertTo( dist8u1, dist8u1.type(), 1, 0 )
    dist32s.convertTo( dist32s, dist32s.type(), -1, 0 )
    bitwise_and( dist32s, Scalar.all(255), dist32s )
    dist32s.convertTo( dist8u2, dist8u2.type(), 1, 0 )
    merge( (dist8u1, dist8u2, dist8u2), dist8u )
    imshow( wndname, dist8u )


if __name__ == "__main__":
    edge_thresh = 100

    filename = "stuff.jpg"
    if len(sys.argv) > 1:
        filename = sys.argv[1]

    gray = imread( filename, 0 )
    if gray.empty():
        print("Failed to load %s" % filename)
        sys.exit(-1)

    # Create the output image
    dist = Mat(gray.size(), CV_32FC1 )
    dist8u1 = gray.clone()
    dist8u2 = gray.clone()
    dist8u = Mat(gray.size(), CV_8UC3 )
    dist32s = Mat(gray.size(), CV_32SC1 )

    # Convert to grayscale
    edge = gray.clone()
    
    # create the window
    namedWindow( wndname, CV_WINDOW_AUTOSIZE )

    # create a toolbar 
    createTrackbar( tbarname, wndname, edge_thresh, 255, on_trackbar )

    # Show the image
    on_trackbar(edge_thresh, None)

    # Wait for a key stroke; the same function arranges events processing
    waitKey(0)



#!/usr/bin/env python
# OpenCV's Python demo
# -- adapted by Minh-Tri Pham to work with pyopencv

import sys

# import the necessary things for OpenCV
from pyopencv import *

#############################################################################
# so, here is the main part of the program

if __name__ == '__main__':

    # a small welcome
    print("OpenCV Python capture video")

    # first, create the necessary window
    namedWindow('Camera', CV_WINDOW_AUTOSIZE)

    # move the new window to a better place
    moveWindow('Camera', 10, 10)

    try:
        # try to get the device number from the command line
        device = int(sys.argv [1])

        # got it ! so remove it from the arguments
        del sys.argv [1]
    except (IndexError, ValueError):
        # no device number on the command line, assume we want the 1st device
        device = 0

    if len (sys.argv) == 1:
        # no argument on the command line, try to use the camera
        capture = VideoCapture(device)
    else:
        # we have an argument on the command line,
        # we can assume this is a file name, so open it
        capture = VideoCapture(sys.argv [1])            

    # check that capture device is OK
    if not capture.isOpened():
        print("Error opening capture device")
        sys.exit (1)

    # create a Mat instance
    frame = Mat()

    # capture the 1st frame to get some propertie on it
    capture >> frame

    # get size of the frame
    frame_size = frame.size()

    # get the frame rate of the capture device
    fps = capture.get(CV_CAP_PROP_FPS)
    if fps == 0:
        # no fps getted, so set it to 30 by default
        fps = 30

    # create the writer
    writer = VideoWriter ("captured.avi", CV_FOURCC('X','v','i','D'), fps, frame_size, True)

    # check the writer is OK
    if not writer.isOpened():
        print("Error opening writer")
        sys.exit (1)
        
    while True:
        # do forever

        # 1. capture the current image
        capture >> frame
        if frame.empty():
            # no image captured... end the processing
            break

        # write the frame to the output file
        writer << frame

        # display the frames to have a visual output
        imshow('Camera', frame)

        # handle events
        if waitKey (5) & 255 == 27:
            # user has press the ESC key, so exit
            break
